@@ -16,16 +16,15 @@
 //
 //
 
-#ifndef GRPC_CORE_LIB_IOMGR_CALL_COMBINER_H
-#define GRPC_CORE_LIB_IOMGR_CALL_COMBINER_H
-
-#include <grpc/support/port_platform.h>
+#ifndef GRPC_SRC_CORE_LIB_IOMGR_CALL_COMBINER_H
+#define GRPC_SRC_CORE_LIB_IOMGR_CALL_COMBINER_H
 
 #include <stddef.h>
 
 #include "absl/container/inlined_vector.h"
 
 #include <grpc/support/atm.h>
+#include <grpc/support/port_platform.h>
 
 #include "src/core/lib/gprpp/mpscq.h"
 #include "src/core/lib/gprpp/ref_counted.h"
@@ -45,8 +44,6 @@
 
 namespace grpc_core {
 
-extern DebugOnlyTraceFlag grpc_call_combiner_trace;
-
 class CallCombiner {
  public:
   CallCombiner();
@@ -168,11 +165,11 @@ class CallCombinerClosureList {
       GRPC_CALL_COMBINER_START(call_combiner, closure.closure, closure.error,
                                closure.reason);
     }
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_call_combiner_trace)) {
+    if (GRPC_TRACE_FLAG_ENABLED(call_combiner)) {
       gpr_log(GPR_INFO,
               "CallCombinerClosureList executing closure while already "
-              "holding call_combiner %p: closure=%p error=%s reason=%s",
-              call_combiner, closures_[0].closure,
+              "holding call_combiner %p: closure=%s error=%s reason=%s",
+              call_combiner, closures_[0].closure->DebugString().c_str(),
               StatusToString(closures_[0].error).c_str(), closures_[0].reason);
     }
     // This will release the call combiner.
@@ -211,4 +208,4 @@ class CallCombinerClosureList {
 
 }  // namespace grpc_core
 
-#endif  // GRPC_CORE_LIB_IOMGR_CALL_COMBINER_H
+#endif  // GRPC_SRC_CORE_LIB_IOMGR_CALL_COMBINER_H
